module.exports="# pack-refs with: peeled \n62c8aea0c531b0ba62dcc72f15d299b280ba9f00 refs/heads/arrayvectors\n846913aea0d42275542aec23e5cbb9df151ba301 refs/heads/feature/attractors\n98ee741299874f0dbc9478a97d7d73258a819060 refs/heads/master\n62c8aea0c531b0ba62dcc72f15d299b280ba9f00 refs/remotes/maxogden/arrayvectors\nefa74713afeffb2d6ed91b2e09b053d689c6faf2 refs/remotes/maxogden/master\n98ee741299874f0dbc9478a97d7d73258a819060 refs/remotes/origin/master\n846913aea0d42275542aec23e5cbb9df151ba301 refs/remotes/richo/features/attractors\n3037af20b9be6415495774e92c34063eea73ce31 refs/remotes/richo/master\nfe3ceaf4f165812ef6c9a83cd9a5e83a5abc20a9 refs/tags/v0.0.3\n^f8f36c9fa55cf6d525d686b1010adf2b3870a4e9\n70c92e69ab0c3953a0752e7ec79ae349027ae1e7 refs/tags/v0.0.4\n^95a0740f93cb22d3d0fc20cc5d616ed12136855f\nd8d196b3298f32976c52712c279c052e1ad72196 refs/tags/v0.0.5\n^efa74713afeffb2d6ed91b2e09b053d689c6faf2\neec4bb7b7a8bd539d54398b6b81fa47c975f0fc1 refs/tags/v0.0.6\n^68c067c6e5910a050416127f6556ad0869343795\na886d739baeae0a8da8823d8c0c6c9452d058837 refs/tags/v0.0.7\n^98ee741299874f0dbc9478a97d7d73258a819060\n"
