export declare class SymlinkError extends Error {
    path: string;
    symlink: string;
    syscall: "symlink";
    code: "TAR_SYMLINK_ERROR";
    constructor(symlink: string, path: string);
    get name(): string;
}
//# sourceMappingURL=symlink-error.d.ts.map