/*
 * WARNING: do not edit!
 * Generated by makefile from include/internal/param_names.h.in
 *
 * Copyright 2023 The OpenSSL Project Authors. All Rights Reserved.
 *
 * Licensed under the Apache License 2.0 (the "License").  You may not use
 * this file except in compliance with the License.  You can obtain a copy
 * in the file LICENSE in the source distribution or at
 * https://www.openssl.org/source/license.html
 */
/* clang-format off */

/* clang-format on */

int ossl_param_find_pidx(const char *s);

/* Parameter name definitions - generated by util/perl/OpenSSL/paramnames.pm */
/* clang-format off */
#define NUM_PIDX 346

#define PIDX_ALG_PARAM_ALGORITHM_ID 0
#define PIDX_ALG_PARAM_ALGORITHM_ID_PARAMS 1
#define PIDX_ALG_PARAM_CIPHER 2
#define PIDX_ALG_PARAM_DIGEST 3
#define PIDX_ALG_PARAM_ENGINE 4
#define PIDX_ALG_PARAM_FIPS_APPROVED_INDICATOR 5
#define PIDX_ALG_PARAM_MAC 6
#define PIDX_ALG_PARAM_PROPERTIES 7
#define PIDX_ASYM_CIPHER_PARAM_DIGEST PIDX_PKEY_PARAM_DIGEST
#define PIDX_ASYM_CIPHER_PARAM_ENGINE PIDX_PKEY_PARAM_ENGINE
#define PIDX_ASYM_CIPHER_PARAM_FIPS_APPROVED_INDICATOR PIDX_ALG_PARAM_FIPS_APPROVED_INDICATOR
#define PIDX_ASYM_CIPHER_PARAM_FIPS_KEY_CHECK PIDX_PKEY_PARAM_FIPS_KEY_CHECK
#define PIDX_ASYM_CIPHER_PARAM_FIPS_RSA_PKCS15_PAD_DISABLED PIDX_PROV_PARAM_RSA_PKCS15_PAD_DISABLED
#define PIDX_ASYM_CIPHER_PARAM_IMPLICIT_REJECTION 8
#define PIDX_ASYM_CIPHER_PARAM_MGF1_DIGEST PIDX_PKEY_PARAM_MGF1_DIGEST
#define PIDX_ASYM_CIPHER_PARAM_MGF1_DIGEST_PROPS PIDX_PKEY_PARAM_MGF1_PROPERTIES
#define PIDX_ASYM_CIPHER_PARAM_OAEP_DIGEST PIDX_ALG_PARAM_DIGEST
#define PIDX_ASYM_CIPHER_PARAM_OAEP_DIGEST_PROPS 9
#define PIDX_ASYM_CIPHER_PARAM_OAEP_LABEL 10
#define PIDX_ASYM_CIPHER_PARAM_PAD_MODE PIDX_PKEY_PARAM_PAD_MODE
#define PIDX_ASYM_CIPHER_PARAM_PROPERTIES PIDX_PKEY_PARAM_PROPERTIES
#define PIDX_ASYM_CIPHER_PARAM_TLS_CLIENT_VERSION 11
#define PIDX_ASYM_CIPHER_PARAM_TLS_NEGOTIATED_VERSION 12
#define PIDX_CAPABILITY_TLS_GROUP_ALG 13
#define PIDX_CAPABILITY_TLS_GROUP_ID 14
#define PIDX_CAPABILITY_TLS_GROUP_IS_KEM 15
#define PIDX_CAPABILITY_TLS_GROUP_MAX_DTLS 16
#define PIDX_CAPABILITY_TLS_GROUP_MAX_TLS 17
#define PIDX_CAPABILITY_TLS_GROUP_MIN_DTLS 18
#define PIDX_CAPABILITY_TLS_GROUP_MIN_TLS 19
#define PIDX_CAPABILITY_TLS_GROUP_NAME 20
#define PIDX_CAPABILITY_TLS_GROUP_NAME_INTERNAL 21
#define PIDX_CAPABILITY_TLS_GROUP_SECURITY_BITS 22
#define PIDX_CAPABILITY_TLS_SIGALG_CODE_POINT 23
#define PIDX_CAPABILITY_TLS_SIGALG_HASH_NAME 24
#define PIDX_CAPABILITY_TLS_SIGALG_HASH_OID 25
#define PIDX_CAPABILITY_TLS_SIGALG_IANA_NAME 26
#define PIDX_CAPABILITY_TLS_SIGALG_KEYTYPE 27
#define PIDX_CAPABILITY_TLS_SIGALG_KEYTYPE_OID 28
#define PIDX_CAPABILITY_TLS_SIGALG_MAX_DTLS 16
#define PIDX_CAPABILITY_TLS_SIGALG_MAX_TLS 17
#define PIDX_CAPABILITY_TLS_SIGALG_MIN_DTLS 18
#define PIDX_CAPABILITY_TLS_SIGALG_MIN_TLS 19
#define PIDX_CAPABILITY_TLS_SIGALG_NAME 29
#define PIDX_CAPABILITY_TLS_SIGALG_OID 30
#define PIDX_CAPABILITY_TLS_SIGALG_SECURITY_BITS 31
#define PIDX_CAPABILITY_TLS_SIGALG_SIG_NAME 32
#define PIDX_CAPABILITY_TLS_SIGALG_SIG_OID 33
#define PIDX_CIPHER_PARAM_AEAD 34
#define PIDX_CIPHER_PARAM_AEAD_IVLEN PIDX_CIPHER_PARAM_IVLEN
#define PIDX_CIPHER_PARAM_AEAD_IV_GENERATED 35
#define PIDX_CIPHER_PARAM_AEAD_MAC_KEY 36
#define PIDX_CIPHER_PARAM_AEAD_TAG 37
#define PIDX_CIPHER_PARAM_AEAD_TAGLEN 38
#define PIDX_CIPHER_PARAM_AEAD_TLS1_AAD 39
#define PIDX_CIPHER_PARAM_AEAD_TLS1_AAD_PAD 40
#define PIDX_CIPHER_PARAM_AEAD_TLS1_GET_IV_GEN 41
#define PIDX_CIPHER_PARAM_AEAD_TLS1_IV_FIXED 42
#define PIDX_CIPHER_PARAM_AEAD_TLS1_SET_IV_INV 43
#define PIDX_CIPHER_PARAM_ALGORITHM_ID PIDX_ALG_PARAM_ALGORITHM_ID
#define PIDX_CIPHER_PARAM_ALGORITHM_ID_PARAMS PIDX_ALG_PARAM_ALGORITHM_ID_PARAMS
#define PIDX_CIPHER_PARAM_ALGORITHM_ID_PARAMS_OLD 44
#define PIDX_CIPHER_PARAM_BLOCK_SIZE 45
#define PIDX_CIPHER_PARAM_CTS 46
#define PIDX_CIPHER_PARAM_CTS_MODE 47
#define PIDX_CIPHER_PARAM_CUSTOM_IV 48
#define PIDX_CIPHER_PARAM_DECRYPT_ONLY 49
#define PIDX_CIPHER_PARAM_FIPS_APPROVED_INDICATOR PIDX_ALG_PARAM_FIPS_APPROVED_INDICATOR
#define PIDX_CIPHER_PARAM_FIPS_ENCRYPT_CHECK 50
#define PIDX_CIPHER_PARAM_HAS_RAND_KEY 51
#define PIDX_CIPHER_PARAM_IV 52
#define PIDX_CIPHER_PARAM_IVLEN 53
#define PIDX_CIPHER_PARAM_KEYLEN 54
#define PIDX_CIPHER_PARAM_MODE 55
#define PIDX_CIPHER_PARAM_NUM 56
#define PIDX_CIPHER_PARAM_PADDING 57
#define PIDX_CIPHER_PARAM_PIPELINE_AEAD_TAG 58
#define PIDX_CIPHER_PARAM_RANDOM_KEY 59
#define PIDX_CIPHER_PARAM_RC2_KEYBITS 60
#define PIDX_CIPHER_PARAM_ROUNDS 61
#define PIDX_CIPHER_PARAM_SPEED 62
#define PIDX_CIPHER_PARAM_TLS1_MULTIBLOCK 63
#define PIDX_CIPHER_PARAM_TLS1_MULTIBLOCK_AAD 64
#define PIDX_CIPHER_PARAM_TLS1_MULTIBLOCK_AAD_PACKLEN 65
#define PIDX_CIPHER_PARAM_TLS1_MULTIBLOCK_ENC 66
#define PIDX_CIPHER_PARAM_TLS1_MULTIBLOCK_ENC_IN 67
#define PIDX_CIPHER_PARAM_TLS1_MULTIBLOCK_ENC_LEN 68
#define PIDX_CIPHER_PARAM_TLS1_MULTIBLOCK_INTERLEAVE 69
#define PIDX_CIPHER_PARAM_TLS1_MULTIBLOCK_MAX_BUFSIZE 70
#define PIDX_CIPHER_PARAM_TLS1_MULTIBLOCK_MAX_SEND_FRAGMENT 71
#define PIDX_CIPHER_PARAM_TLS_MAC 72
#define PIDX_CIPHER_PARAM_TLS_MAC_SIZE 73
#define PIDX_CIPHER_PARAM_TLS_VERSION 74
#define PIDX_CIPHER_PARAM_UPDATED_IV 75
#define PIDX_CIPHER_PARAM_USE_BITS 76
#define PIDX_CIPHER_PARAM_XTS_STANDARD 77
#define PIDX_DECODER_PARAM_PROPERTIES PIDX_ALG_PARAM_PROPERTIES
#define PIDX_DIGEST_PARAM_ALGID_ABSENT 78
#define PIDX_DIGEST_PARAM_BLOCK_SIZE 45
#define PIDX_DIGEST_PARAM_MICALG 79
#define PIDX_DIGEST_PARAM_PAD_TYPE 80
#define PIDX_DIGEST_PARAM_SIZE 81
#define PIDX_DIGEST_PARAM_SSL3_MS 82
#define PIDX_DIGEST_PARAM_XOF 83
#define PIDX_DIGEST_PARAM_XOFLEN 84
#define PIDX_DRBG_PARAM_CIPHER PIDX_ALG_PARAM_CIPHER
#define PIDX_DRBG_PARAM_DIGEST PIDX_ALG_PARAM_DIGEST
#define PIDX_DRBG_PARAM_ENTROPY_REQUIRED 85
#define PIDX_DRBG_PARAM_FIPS_APPROVED_INDICATOR PIDX_ALG_PARAM_FIPS_APPROVED_INDICATOR
#define PIDX_DRBG_PARAM_FIPS_DIGEST_CHECK PIDX_PKEY_PARAM_FIPS_DIGEST_CHECK
#define PIDX_DRBG_PARAM_MAC PIDX_ALG_PARAM_MAC
#define PIDX_DRBG_PARAM_MAX_ADINLEN 86
#define PIDX_DRBG_PARAM_MAX_ENTROPYLEN 87
#define PIDX_DRBG_PARAM_MAX_LENGTH 88
#define PIDX_DRBG_PARAM_MAX_NONCELEN 89
#define PIDX_DRBG_PARAM_MAX_PERSLEN 90
#define PIDX_DRBG_PARAM_MIN_ENTROPYLEN 91
#define PIDX_DRBG_PARAM_MIN_LENGTH 92
#define PIDX_DRBG_PARAM_MIN_NONCELEN 93
#define PIDX_DRBG_PARAM_PREDICTION_RESISTANCE 94
#define PIDX_DRBG_PARAM_PROPERTIES PIDX_ALG_PARAM_PROPERTIES
#define PIDX_DRBG_PARAM_RANDOM_DATA 95
#define PIDX_DRBG_PARAM_RESEED_COUNTER 96
#define PIDX_DRBG_PARAM_RESEED_REQUESTS 97
#define PIDX_DRBG_PARAM_RESEED_TIME 98
#define PIDX_DRBG_PARAM_RESEED_TIME_INTERVAL 99
#define PIDX_DRBG_PARAM_SIZE 81
#define PIDX_DRBG_PARAM_USE_DF 100
#define PIDX_ENCODER_PARAM_CIPHER PIDX_ALG_PARAM_CIPHER
#define PIDX_ENCODER_PARAM_ENCRYPT_LEVEL 101
#define PIDX_ENCODER_PARAM_PROPERTIES PIDX_ALG_PARAM_PROPERTIES
#define PIDX_ENCODER_PARAM_SAVE_PARAMETERS 102
#define PIDX_EXCHANGE_PARAM_EC_ECDH_COFACTOR_MODE 103
#define PIDX_EXCHANGE_PARAM_FIPS_APPROVED_INDICATOR PIDX_ALG_PARAM_FIPS_APPROVED_INDICATOR
#define PIDX_EXCHANGE_PARAM_FIPS_DIGEST_CHECK PIDX_PKEY_PARAM_FIPS_DIGEST_CHECK
#define PIDX_EXCHANGE_PARAM_FIPS_ECDH_COFACTOR_CHECK PIDX_PROV_PARAM_ECDH_COFACTOR_CHECK
#define PIDX_EXCHANGE_PARAM_FIPS_KEY_CHECK PIDX_PKEY_PARAM_FIPS_KEY_CHECK
#define PIDX_EXCHANGE_PARAM_KDF_DIGEST 104
#define PIDX_EXCHANGE_PARAM_KDF_DIGEST_PROPS 105
#define PIDX_EXCHANGE_PARAM_KDF_OUTLEN 106
#define PIDX_EXCHANGE_PARAM_KDF_TYPE 107
#define PIDX_EXCHANGE_PARAM_KDF_UKM 108
#define PIDX_EXCHANGE_PARAM_PAD 109
#define PIDX_GEN_PARAM_ITERATION 110
#define PIDX_GEN_PARAM_POTENTIAL 111
#define PIDX_KDF_PARAM_ARGON2_AD 112
#define PIDX_KDF_PARAM_ARGON2_LANES 113
#define PIDX_KDF_PARAM_ARGON2_MEMCOST 114
#define PIDX_KDF_PARAM_ARGON2_VERSION 115
#define PIDX_KDF_PARAM_CEK_ALG 116
#define PIDX_KDF_PARAM_CIPHER PIDX_ALG_PARAM_CIPHER
#define PIDX_KDF_PARAM_CONSTANT 117
#define PIDX_KDF_PARAM_DATA 118
#define PIDX_KDF_PARAM_DIGEST PIDX_ALG_PARAM_DIGEST
#define PIDX_KDF_PARAM_EARLY_CLEAN 119
#define PIDX_KDF_PARAM_FIPS_APPROVED_INDICATOR PIDX_ALG_PARAM_FIPS_APPROVED_INDICATOR
#define PIDX_KDF_PARAM_FIPS_DIGEST_CHECK PIDX_PKEY_PARAM_FIPS_DIGEST_CHECK
#define PIDX_KDF_PARAM_FIPS_EMS_CHECK 120
#define PIDX_KDF_PARAM_FIPS_KEY_CHECK PIDX_PKEY_PARAM_FIPS_KEY_CHECK
#define PIDX_KDF_PARAM_HMACDRBG_ENTROPY 121
#define PIDX_KDF_PARAM_HMACDRBG_NONCE 122
#define PIDX_KDF_PARAM_INFO 123
#define PIDX_KDF_PARAM_ITER 124
#define PIDX_KDF_PARAM_KBKDF_R 125
#define PIDX_KDF_PARAM_KBKDF_USE_L 126
#define PIDX_KDF_PARAM_KBKDF_USE_SEPARATOR 127
#define PIDX_KDF_PARAM_KEY 128
#define PIDX_KDF_PARAM_LABEL 129
#define PIDX_KDF_PARAM_MAC PIDX_ALG_PARAM_MAC
#define PIDX_KDF_PARAM_MAC_SIZE 130
#define PIDX_KDF_PARAM_MODE 55
#define PIDX_KDF_PARAM_PASSWORD 131
#define PIDX_KDF_PARAM_PKCS12_ID 132
#define PIDX_KDF_PARAM_PKCS5 133
#define PIDX_KDF_PARAM_PREFIX 134
#define PIDX_KDF_PARAM_PROPERTIES PIDX_ALG_PARAM_PROPERTIES
#define PIDX_KDF_PARAM_SALT 135
#define PIDX_KDF_PARAM_SCRYPT_MAXMEM 136
#define PIDX_KDF_PARAM_SCRYPT_N 137
#define PIDX_KDF_PARAM_SCRYPT_P 138
#define PIDX_KDF_PARAM_SCRYPT_R 125
#define PIDX_KDF_PARAM_SECRET 139
#define PIDX_KDF_PARAM_SEED 140
#define PIDX_KDF_PARAM_SIZE 81
#define PIDX_KDF_PARAM_SSHKDF_SESSION_ID 141
#define PIDX_KDF_PARAM_SSHKDF_TYPE 142
#define PIDX_KDF_PARAM_SSHKDF_XCGHASH 143
#define PIDX_KDF_PARAM_THREADS 144
#define PIDX_KDF_PARAM_UKM 145
#define PIDX_KDF_PARAM_X942_ACVPINFO 146
#define PIDX_KDF_PARAM_X942_PARTYUINFO 147
#define PIDX_KDF_PARAM_X942_PARTYVINFO 148
#define PIDX_KDF_PARAM_X942_SUPP_PRIVINFO 149
#define PIDX_KDF_PARAM_X942_SUPP_PUBINFO 150
#define PIDX_KDF_PARAM_X942_USE_KEYBITS 151
#define PIDX_KEM_PARAM_FIPS_APPROVED_INDICATOR PIDX_ALG_PARAM_FIPS_APPROVED_INDICATOR
#define PIDX_KEM_PARAM_FIPS_KEY_CHECK PIDX_PKEY_PARAM_FIPS_KEY_CHECK
#define PIDX_KEM_PARAM_IKME 152
#define PIDX_KEM_PARAM_OPERATION 153
#define PIDX_LIBSSL_RECORD_LAYER_PARAM_BLOCK_PADDING 154
#define PIDX_LIBSSL_RECORD_LAYER_PARAM_HS_PADDING 155
#define PIDX_LIBSSL_RECORD_LAYER_PARAM_MAX_EARLY_DATA 156
#define PIDX_LIBSSL_RECORD_LAYER_PARAM_MAX_FRAG_LEN 157
#define PIDX_LIBSSL_RECORD_LAYER_PARAM_MODE 55
#define PIDX_LIBSSL_RECORD_LAYER_PARAM_OPTIONS 158
#define PIDX_LIBSSL_RECORD_LAYER_PARAM_READ_AHEAD 159
#define PIDX_LIBSSL_RECORD_LAYER_PARAM_STREAM_MAC 160
#define PIDX_LIBSSL_RECORD_LAYER_PARAM_TLSTREE 161
#define PIDX_LIBSSL_RECORD_LAYER_PARAM_USE_ETM 162
#define PIDX_LIBSSL_RECORD_LAYER_READ_BUFFER_LEN 163
#define PIDX_MAC_PARAM_BLOCK_SIZE 164
#define PIDX_MAC_PARAM_CIPHER PIDX_ALG_PARAM_CIPHER
#define PIDX_MAC_PARAM_CUSTOM 165
#define PIDX_MAC_PARAM_C_ROUNDS 166
#define PIDX_MAC_PARAM_DIGEST PIDX_ALG_PARAM_DIGEST
#define PIDX_MAC_PARAM_DIGEST_NOINIT 167
#define PIDX_MAC_PARAM_DIGEST_ONESHOT 168
#define PIDX_MAC_PARAM_D_ROUNDS 169
#define PIDX_MAC_PARAM_FIPS_APPROVED_INDICATOR PIDX_ALG_PARAM_FIPS_APPROVED_INDICATOR
#define PIDX_MAC_PARAM_FIPS_KEY_CHECK PIDX_PKEY_PARAM_FIPS_KEY_CHECK
#define PIDX_MAC_PARAM_FIPS_NO_SHORT_MAC PIDX_PROV_PARAM_NO_SHORT_MAC
#define PIDX_MAC_PARAM_IV 52
#define PIDX_MAC_PARAM_KEY 128
#define PIDX_MAC_PARAM_PROPERTIES PIDX_ALG_PARAM_PROPERTIES
#define PIDX_MAC_PARAM_SALT 135
#define PIDX_MAC_PARAM_SIZE 81
#define PIDX_MAC_PARAM_TLS_DATA_SIZE 170
#define PIDX_MAC_PARAM_XOF 83
#define PIDX_OBJECT_PARAM_DATA 118
#define PIDX_OBJECT_PARAM_DATA_STRUCTURE 171
#define PIDX_OBJECT_PARAM_DATA_TYPE 172
#define PIDX_OBJECT_PARAM_DESC 173
#define PIDX_OBJECT_PARAM_INPUT_TYPE 174
#define PIDX_OBJECT_PARAM_REFERENCE 175
#define PIDX_OBJECT_PARAM_TYPE 142
#define PIDX_PASSPHRASE_PARAM_INFO 123
#define PIDX_PKEY_PARAM_ALGORITHM_ID PIDX_ALG_PARAM_ALGORITHM_ID
#define PIDX_PKEY_PARAM_ALGORITHM_ID_PARAMS PIDX_ALG_PARAM_ALGORITHM_ID_PARAMS
#define PIDX_PKEY_PARAM_BITS 176
#define PIDX_PKEY_PARAM_CIPHER PIDX_ALG_PARAM_CIPHER
#define PIDX_PKEY_PARAM_DEFAULT_DIGEST 177
#define PIDX_PKEY_PARAM_DHKEM_IKM 178
#define PIDX_PKEY_PARAM_DH_GENERATOR 179
#define PIDX_PKEY_PARAM_DH_PRIV_LEN 180
#define PIDX_PKEY_PARAM_DIGEST PIDX_ALG_PARAM_DIGEST
#define PIDX_PKEY_PARAM_DIGEST_SIZE 181
#define PIDX_PKEY_PARAM_DIST_ID 182
#define PIDX_PKEY_PARAM_EC_A 183
#define PIDX_PKEY_PARAM_EC_B 184
#define PIDX_PKEY_PARAM_EC_CHAR2_M 185
#define PIDX_PKEY_PARAM_EC_CHAR2_PP_K1 186
#define PIDX_PKEY_PARAM_EC_CHAR2_PP_K2 187
#define PIDX_PKEY_PARAM_EC_CHAR2_PP_K3 188
#define PIDX_PKEY_PARAM_EC_CHAR2_TP_BASIS 189
#define PIDX_PKEY_PARAM_EC_CHAR2_TYPE 190
#define PIDX_PKEY_PARAM_EC_COFACTOR 191
#define PIDX_PKEY_PARAM_EC_DECODED_FROM_EXPLICIT_PARAMS 192
#define PIDX_PKEY_PARAM_EC_ENCODING 193
#define PIDX_PKEY_PARAM_EC_FIELD_TYPE 194
#define PIDX_PKEY_PARAM_EC_GENERATOR 195
#define PIDX_PKEY_PARAM_EC_GROUP_CHECK_TYPE 196
#define PIDX_PKEY_PARAM_EC_INCLUDE_PUBLIC 197
#define PIDX_PKEY_PARAM_EC_ORDER 198
#define PIDX_PKEY_PARAM_EC_P 138
#define PIDX_PKEY_PARAM_EC_POINT_CONVERSION_FORMAT 199
#define PIDX_PKEY_PARAM_EC_PUB_X 200
#define PIDX_PKEY_PARAM_EC_PUB_Y 201
#define PIDX_PKEY_PARAM_EC_SEED 140
#define PIDX_PKEY_PARAM_ENCODED_PUBLIC_KEY 202
#define PIDX_PKEY_PARAM_ENGINE PIDX_ALG_PARAM_ENGINE
#define PIDX_PKEY_PARAM_FFC_COFACTOR 203
#define PIDX_PKEY_PARAM_FFC_DIGEST PIDX_PKEY_PARAM_DIGEST
#define PIDX_PKEY_PARAM_FFC_DIGEST_PROPS PIDX_PKEY_PARAM_PROPERTIES
#define PIDX_PKEY_PARAM_FFC_G 204
#define PIDX_PKEY_PARAM_FFC_GINDEX 205
#define PIDX_PKEY_PARAM_FFC_H 206
#define PIDX_PKEY_PARAM_FFC_P 138
#define PIDX_PKEY_PARAM_FFC_PBITS 207
#define PIDX_PKEY_PARAM_FFC_PCOUNTER 208
#define PIDX_PKEY_PARAM_FFC_Q 209
#define PIDX_PKEY_PARAM_FFC_QBITS 210
#define PIDX_PKEY_PARAM_FFC_SEED 140
#define PIDX_PKEY_PARAM_FFC_TYPE 142
#define PIDX_PKEY_PARAM_FFC_VALIDATE_G 211
#define PIDX_PKEY_PARAM_FFC_VALIDATE_LEGACY 212
#define PIDX_PKEY_PARAM_FFC_VALIDATE_PQ 213
#define PIDX_PKEY_PARAM_FIPS_APPROVED_INDICATOR PIDX_ALG_PARAM_FIPS_APPROVED_INDICATOR
#define PIDX_PKEY_PARAM_FIPS_DIGEST_CHECK 214
#define PIDX_PKEY_PARAM_FIPS_KEY_CHECK 215
#define PIDX_PKEY_PARAM_FIPS_SIGN_CHECK 216
#define PIDX_PKEY_PARAM_GROUP_NAME 217
#define PIDX_PKEY_PARAM_IMPLICIT_REJECTION 8
#define PIDX_PKEY_PARAM_MANDATORY_DIGEST 218
#define PIDX_PKEY_PARAM_MASKGENFUNC 219
#define PIDX_PKEY_PARAM_MAX_SIZE 220
#define PIDX_PKEY_PARAM_MGF1_DIGEST 221
#define PIDX_PKEY_PARAM_MGF1_PROPERTIES 222
#define PIDX_PKEY_PARAM_ML_DSA_INPUT_FORMATS 223
#define PIDX_PKEY_PARAM_ML_DSA_OUTPUT_FORMATS 224
#define PIDX_PKEY_PARAM_ML_DSA_PREFER_SEED 225
#define PIDX_PKEY_PARAM_ML_DSA_RETAIN_SEED 226
#define PIDX_PKEY_PARAM_ML_DSA_SEED 140
#define PIDX_PKEY_PARAM_ML_KEM_IMPORT_PCT_TYPE 227
#define PIDX_PKEY_PARAM_ML_KEM_INPUT_FORMATS 228
#define PIDX_PKEY_PARAM_ML_KEM_OUTPUT_FORMATS 229
#define PIDX_PKEY_PARAM_ML_KEM_PREFER_SEED 230
#define PIDX_PKEY_PARAM_ML_KEM_RETAIN_SEED 231
#define PIDX_PKEY_PARAM_ML_KEM_SEED 140
#define PIDX_PKEY_PARAM_PAD_MODE 232
#define PIDX_PKEY_PARAM_PRIV_KEY 233
#define PIDX_PKEY_PARAM_PROPERTIES PIDX_ALG_PARAM_PROPERTIES
#define PIDX_PKEY_PARAM_PUB_KEY 234
#define PIDX_PKEY_PARAM_RSA_BITS PIDX_PKEY_PARAM_BITS
#define PIDX_PKEY_PARAM_RSA_COEFFICIENT 235
#define PIDX_PKEY_PARAM_RSA_COEFFICIENT1 236
#define PIDX_PKEY_PARAM_RSA_COEFFICIENT2 237
#define PIDX_PKEY_PARAM_RSA_COEFFICIENT3 238
#define PIDX_PKEY_PARAM_RSA_COEFFICIENT4 239
#define PIDX_PKEY_PARAM_RSA_COEFFICIENT5 240
#define PIDX_PKEY_PARAM_RSA_COEFFICIENT6 241
#define PIDX_PKEY_PARAM_RSA_COEFFICIENT7 242
#define PIDX_PKEY_PARAM_RSA_COEFFICIENT8 243
#define PIDX_PKEY_PARAM_RSA_COEFFICIENT9 244
#define PIDX_PKEY_PARAM_RSA_D 245
#define PIDX_PKEY_PARAM_RSA_DERIVE_FROM_PQ 246
#define PIDX_PKEY_PARAM_RSA_DIGEST PIDX_PKEY_PARAM_DIGEST
#define PIDX_PKEY_PARAM_RSA_DIGEST_PROPS PIDX_PKEY_PARAM_PROPERTIES
#define PIDX_PKEY_PARAM_RSA_E 247
#define PIDX_PKEY_PARAM_RSA_EXPONENT 248
#define PIDX_PKEY_PARAM_RSA_EXPONENT1 249
#define PIDX_PKEY_PARAM_RSA_EXPONENT10 250
#define PIDX_PKEY_PARAM_RSA_EXPONENT2 251
#define PIDX_PKEY_PARAM_RSA_EXPONENT3 252
#define PIDX_PKEY_PARAM_RSA_EXPONENT4 253
#define PIDX_PKEY_PARAM_RSA_EXPONENT5 254
#define PIDX_PKEY_PARAM_RSA_EXPONENT6 255
#define PIDX_PKEY_PARAM_RSA_EXPONENT7 256
#define PIDX_PKEY_PARAM_RSA_EXPONENT8 257
#define PIDX_PKEY_PARAM_RSA_EXPONENT9 258
#define PIDX_PKEY_PARAM_RSA_FACTOR 259
#define PIDX_PKEY_PARAM_RSA_FACTOR1 260
#define PIDX_PKEY_PARAM_RSA_FACTOR10 261
#define PIDX_PKEY_PARAM_RSA_FACTOR2 262
#define PIDX_PKEY_PARAM_RSA_FACTOR3 263
#define PIDX_PKEY_PARAM_RSA_FACTOR4 264
#define PIDX_PKEY_PARAM_RSA_FACTOR5 265
#define PIDX_PKEY_PARAM_RSA_FACTOR6 266
#define PIDX_PKEY_PARAM_RSA_FACTOR7 267
#define PIDX_PKEY_PARAM_RSA_FACTOR8 268
#define PIDX_PKEY_PARAM_RSA_FACTOR9 269
#define PIDX_PKEY_PARAM_RSA_MASKGENFUNC PIDX_PKEY_PARAM_MASKGENFUNC
#define PIDX_PKEY_PARAM_RSA_MGF1_DIGEST PIDX_PKEY_PARAM_MGF1_DIGEST
#define PIDX_PKEY_PARAM_RSA_N 137
#define PIDX_PKEY_PARAM_RSA_PRIMES 270
#define PIDX_PKEY_PARAM_RSA_PSS_SALTLEN 271
#define PIDX_PKEY_PARAM_RSA_TEST_P1 272
#define PIDX_PKEY_PARAM_RSA_TEST_P2 273
#define PIDX_PKEY_PARAM_RSA_TEST_Q1 274
#define PIDX_PKEY_PARAM_RSA_TEST_Q2 275
#define PIDX_PKEY_PARAM_RSA_TEST_XP 276
#define PIDX_PKEY_PARAM_RSA_TEST_XP1 277
#define PIDX_PKEY_PARAM_RSA_TEST_XP2 278
#define PIDX_PKEY_PARAM_RSA_TEST_XQ 279
#define PIDX_PKEY_PARAM_RSA_TEST_XQ1 280
#define PIDX_PKEY_PARAM_RSA_TEST_XQ2 281
#define PIDX_PKEY_PARAM_SECURITY_BITS 282
#define PIDX_PKEY_PARAM_SLH_DSA_SEED 140
#define PIDX_PKEY_PARAM_USE_COFACTOR_ECDH PIDX_PKEY_PARAM_USE_COFACTOR_FLAG
#define PIDX_PKEY_PARAM_USE_COFACTOR_FLAG 283
#define PIDX_PROV_PARAM_BUILDINFO 284
#define PIDX_PROV_PARAM_CORE_MODULE_FILENAME 285
#define PIDX_PROV_PARAM_CORE_PROV_NAME 286
#define PIDX_PROV_PARAM_CORE_VERSION 287
#define PIDX_PROV_PARAM_DRBG_TRUNC_DIGEST 288
#define PIDX_PROV_PARAM_DSA_SIGN_DISABLED 289
#define PIDX_PROV_PARAM_ECDH_COFACTOR_CHECK 290
#define PIDX_PROV_PARAM_HKDF_DIGEST_CHECK 291
#define PIDX_PROV_PARAM_HKDF_KEY_CHECK 292
#define PIDX_PROV_PARAM_HMAC_KEY_CHECK 293
#define PIDX_PROV_PARAM_KBKDF_KEY_CHECK 294
#define PIDX_PROV_PARAM_KMAC_KEY_CHECK 295
#define PIDX_PROV_PARAM_NAME 296
#define PIDX_PROV_PARAM_NO_SHORT_MAC 297
#define PIDX_PROV_PARAM_PBKDF2_LOWER_BOUND_CHECK 298
#define PIDX_PROV_PARAM_RSA_PKCS15_PAD_DISABLED 299
#define PIDX_PROV_PARAM_RSA_PSS_SALTLEN_CHECK 300
#define PIDX_PROV_PARAM_RSA_SIGN_X931_PAD_DISABLED 301
#define PIDX_PROV_PARAM_SECURITY_CHECKS 302
#define PIDX_PROV_PARAM_SELF_TEST_DESC 303
#define PIDX_PROV_PARAM_SELF_TEST_PHASE 304
#define PIDX_PROV_PARAM_SELF_TEST_TYPE 305
#define PIDX_PROV_PARAM_SIGNATURE_DIGEST_CHECK 306
#define PIDX_PROV_PARAM_SSHKDF_DIGEST_CHECK 307
#define PIDX_PROV_PARAM_SSHKDF_KEY_CHECK 308
#define PIDX_PROV_PARAM_SSKDF_DIGEST_CHECK 309
#define PIDX_PROV_PARAM_SSKDF_KEY_CHECK 310
#define PIDX_PROV_PARAM_STATUS 311
#define PIDX_PROV_PARAM_TDES_ENCRYPT_DISABLED 312
#define PIDX_PROV_PARAM_TLS13_KDF_DIGEST_CHECK 313
#define PIDX_PROV_PARAM_TLS13_KDF_KEY_CHECK 314
#define PIDX_PROV_PARAM_TLS1_PRF_DIGEST_CHECK 315
#define PIDX_PROV_PARAM_TLS1_PRF_EMS_CHECK 316
#define PIDX_PROV_PARAM_TLS1_PRF_KEY_CHECK 317
#define PIDX_PROV_PARAM_VERSION 115
#define PIDX_PROV_PARAM_X942KDF_KEY_CHECK 318
#define PIDX_PROV_PARAM_X963KDF_DIGEST_CHECK 319
#define PIDX_PROV_PARAM_X963KDF_KEY_CHECK 320
#define PIDX_RAND_PARAM_FIPS_APPROVED_INDICATOR PIDX_ALG_PARAM_FIPS_APPROVED_INDICATOR
#define PIDX_RAND_PARAM_GENERATE 321
#define PIDX_RAND_PARAM_MAX_REQUEST 322
#define PIDX_RAND_PARAM_STATE 323
#define PIDX_RAND_PARAM_STRENGTH 324
#define PIDX_RAND_PARAM_TEST_ENTROPY 325
#define PIDX_RAND_PARAM_TEST_NONCE 326
#define PIDX_SIGNATURE_PARAM_ADD_RANDOM 327
#define PIDX_SIGNATURE_PARAM_ALGORITHM_ID PIDX_PKEY_PARAM_ALGORITHM_ID
#define PIDX_SIGNATURE_PARAM_ALGORITHM_ID_PARAMS PIDX_PKEY_PARAM_ALGORITHM_ID_PARAMS
#define PIDX_SIGNATURE_PARAM_CONTEXT_STRING 328
#define PIDX_SIGNATURE_PARAM_DETERMINISTIC 329
#define PIDX_SIGNATURE_PARAM_DIGEST PIDX_PKEY_PARAM_DIGEST
#define PIDX_SIGNATURE_PARAM_DIGEST_SIZE PIDX_PKEY_PARAM_DIGEST_SIZE
#define PIDX_SIGNATURE_PARAM_FIPS_APPROVED_INDICATOR PIDX_ALG_PARAM_FIPS_APPROVED_INDICATOR
#define PIDX_SIGNATURE_PARAM_FIPS_DIGEST_CHECK PIDX_PKEY_PARAM_FIPS_DIGEST_CHECK
#define PIDX_SIGNATURE_PARAM_FIPS_KEY_CHECK PIDX_PKEY_PARAM_FIPS_KEY_CHECK
#define PIDX_SIGNATURE_PARAM_FIPS_RSA_PSS_SALTLEN_CHECK 300
#define PIDX_SIGNATURE_PARAM_FIPS_SIGN_CHECK PIDX_PKEY_PARAM_FIPS_SIGN_CHECK
#define PIDX_SIGNATURE_PARAM_FIPS_SIGN_X931_PAD_CHECK 330
#define PIDX_SIGNATURE_PARAM_FIPS_VERIFY_MESSAGE 331
#define PIDX_SIGNATURE_PARAM_INSTANCE 332
#define PIDX_SIGNATURE_PARAM_KAT 333
#define PIDX_SIGNATURE_PARAM_MESSAGE_ENCODING 334
#define PIDX_SIGNATURE_PARAM_MGF1_DIGEST PIDX_PKEY_PARAM_MGF1_DIGEST
#define PIDX_SIGNATURE_PARAM_MGF1_PROPERTIES PIDX_PKEY_PARAM_MGF1_PROPERTIES
#define PIDX_SIGNATURE_PARAM_MU 335
#define PIDX_SIGNATURE_PARAM_NONCE_TYPE 336
#define PIDX_SIGNATURE_PARAM_PAD_MODE PIDX_PKEY_PARAM_PAD_MODE
#define PIDX_SIGNATURE_PARAM_PROPERTIES PIDX_PKEY_PARAM_PROPERTIES
#define PIDX_SIGNATURE_PARAM_PSS_SALTLEN 271
#define PIDX_SIGNATURE_PARAM_SIGNATURE 337
#define PIDX_SIGNATURE_PARAM_TEST_ENTROPY 338
#define PIDX_SKEY_PARAM_KEY_LENGTH 339
#define PIDX_SKEY_PARAM_RAW_BYTES 340
#define PIDX_STORE_PARAM_ALIAS 341
#define PIDX_STORE_PARAM_DIGEST 3
#define PIDX_STORE_PARAM_EXPECT 342
#define PIDX_STORE_PARAM_FINGERPRINT 343
#define PIDX_STORE_PARAM_INPUT_TYPE 174
#define PIDX_STORE_PARAM_ISSUER 296
#define PIDX_STORE_PARAM_PROPERTIES 7
#define PIDX_STORE_PARAM_SERIAL 344
#define PIDX_STORE_PARAM_SUBJECT 345
/* clang-format on */
